-- Author      : Nogarder
-- Create Date : 2015/3/2 1:48:24

local RS_Template_SetSpellButton=function(button,spellid,spellcooldown,checked)
	local _,_,icon =GetSpellInfo(spellid)
	button.Label:SetText(spellcooldown)
	button.Texture:SetTexture(icon)
	if(checked)then
		RS.Template.ShowLinerBorder(button)
	else
		RS.Template.HideLinerBorder(button)
	end
	
	button.SpellID=spellid
	button.SpellCoolDown=spellcooldown
end

local RS_Template_ClearSpellButton=function(button)
	button.Label:SetText("+")
	button.Texture:SetTexture(nil)
	RS.Template.HideLinerBorder(button)
	button.SpellID=nil
	button.SpellCoolDown=nil
end

local RS_Template_CreateSpellList=function(parent,width,height,buttonwidth)
	local frame=RS.Template.CreateFrame(parent,width,height)

	local CreateSpellButton=function()
		local button=RS.Template.CreateButton(frame,buttonwidth-5,buttonwidth-5)
		button.Label:SetFont(RS.TextFont,10,RS.TextMode)
		button:HookScript("OnMouseUp",function(self,mousebutton)
			if(self.SpellID==nil)then
				RS.SetPage("CustomSpell",parent:GetParent():GetParent():GetParent().Scroll.Dragger,function(spellid,cooldown)
					if(self.SpellID)then
						frame.SpellList[self.SpellID]=nil
					end
					RS_Template_SetSpellButton(self,spellid,cooldown)
					RS.SpecAndSpells.SetCustomSpell(parent.PlayerData.Name,spellid,cooldown)
					frame.SpellList[spellid]=cooldown
					RS.RunFunc(parent:GetParent().OnSpellAddFunc,parent.PlayerData)
				end)
			elseif(mousebutton=="LeftButton")then
				if(frame.SelectedSpell[self.SpellID])then
					frame.SelectedSpell[self.SpellID]=nil
					RS.Template.HideLinerBorder(self)
					RS.RunFunc(parent:GetParent().OnSpellUncheckFunc,parent.PlayerData)
				else
					frame.SelectedSpell[self.SpellID]=self.SpellCoolDown
					RS.Template.ShowLinerBorder(self)
					RS.RunFunc(parent:GetParent().OnSpellCheckFunc,parent.PlayerData)
				end
			elseif(mousebutton=="RightButton")then
				local spellid=self.SpellID
				if(frame.SelectedSpell[self.SpellID])then
					frame.SelectedSpell[self.SpellID]=nil
					RS.Template.HideLinerBorder(self)
					RS.RunFunc(parent:GetParent().OnSpellUncheckFunc,parent.PlayerData)
				end
				parent.PlayerData.SelectedSpell[self.SpellID]=nil
				RS_Template_ClearSpellButton(self)
				RS.RunFunc(parent:GetParent().OnSpellRemovedFunc,parent.PlayerData)
				--also open the spell look up table
				RS.SetPage("CustomSpell",parent:GetParent():GetParent():GetParent().Scroll.Dragger,function(spellid,cooldown)
					if(self.SpellID)then
						frame.SpellList[self.SpellID]=nil
					end
					RS_Template_SetSpellButton(self,spellid,cooldown)
					RS.SpecAndSpells.SetCustomSpell(parent.PlayerData.Name,spellid,cooldown)
					frame.SpellList[spellid]=cooldown
					RS.RunFunc(parent:GetParent().OnSpellAddFunc,parent.PlayerData)
				end,
				spellid)
			end
		end)

		button:HookScript("OnEnter",function(self)
			if(self.SpellID)then
				GameTooltip:SetOwner(self, "ANCHOR_TOP")
				GameTooltip:SetSpellByID(self.SpellID)
			end
		end)
		button:HookScript("OnLeave",function(self)
			if(GameTooltip:IsOwned(self))then
				GameTooltip:Hide()
			end
		end)

		RS.Template.CreateLinerBorder(button,2)
		RS.Template.HideLinerBorder(button)

		return button
	end

	local count=0
	local framelist={}

	local perrow=floor(width/buttonwidth)
	for i=1,perrow do
		count=count+1
		framelist[i]=CreateSpellButton()
		framelist[i]:SetPoint("LEFT",frame,"LEFT",5+(i-1)*buttonwidth,0)
	end

	frame.FrameList=framelist
	frame.FrameCount=count

	return frame
end

local RS_Template_SetSpellList=function(frame,spelllist,selectedlist)
	local max=frame.FrameCount
	local framelist=	frame.FrameList
	local i=1
	for spellid,cooldown in pairs(spelllist) do
		RS_Template_SetSpellButton(framelist[i],spellid,cooldown,selectedlist[spellid])
		i=i+1
		if(i>max)then
			break
		end
	end
	for j=i,frame.FrameCount do
		RS_Template_ClearSpellButton(framelist[j])
	end
	
	frame.SpellList=spelllist
	frame.SelectedSpell=selectedlist
end

local RS_Template_CreatePlayerFrame=function(frame,index)
	local width=frame:GetWidth()
	local height=frame:GetHeight()
	local buttonheight=frame.ButtonHeight

	local button=RS.Template.CreateButton(frame,width,buttonheight)
	button.Label:ClearAllPoints()
	button.Label:SetFont(RS.TextFont,10,RS.TextMode)
	button.Label:SetPoint("TOPLEFT",button,"TOPLEFT",0,0)
	button.Label:SetWidth(buttonheight)
	button:HookScript("OnMouseUp",function(self)
		RS.RunFunc(frame.OnPlayerClickFunc,self.PlayerData)
	end)

	local halfheight=buttonheight/2

	button.PortraitTexture=button:CreateTexture(nil,"ARTWORK")
	button.PortraitTexture:SetPoint("TOPLEFT",button,"TOPLEFT")
	button.PortraitTexture:SetWidth(buttonheight)
	button.PortraitTexture:SetHeight(buttonheight)

	button.RoleTexture=button:CreateTexture(nil,"OVERLAY")
	button.RoleTexture:SetPoint("BOTTOMLEFT",button,"BOTTOMLEFT")
	button.RoleTexture:SetWidth(halfheight)
	button.RoleTexture:SetHeight(halfheight)
	button.RoleTexture:SetTexture(RS.RoleTexture)
	button.RoleTexture:SetTexCoord(0,0,0,0)

	button.SpecTexture=button:CreateTexture(nil,"OVERLAY")
	button.SpecTexture:SetPoint("BOTTOMLEFT",button.RoleTexture,"BOTTOMRIGHT")
	button.SpecTexture:SetWidth(halfheight)
	button.SpecTexture:SetHeight(halfheight)

	button.SpellList=RS_Template_CreateSpellList(button,width-buttonheight,buttonheight,buttonheight)
	button.SpellList:SetPoint("TOPLEFT",button.PortraitTexture,"TOPRIGHT",0,0)
	
	button:SetPoint("BOTTOMLEFT",frame,"TOPLEFT",0,-index*buttonheight)

	return button
end

local RS_Template_SetPlayerFrame=function(button,playerdata)
	button.PlayerData=playerdata

	RS.SpecAndSpells.SetPortrait(button.PortraitTexture,playerdata.Name)
	button.Label:SetText(RS.GetName(playerdata.Name))
	RS.SetTextColor(button.Label,playerdata.Color)
	RS.SetTexCoord(button.RoleTexture,playerdata.RoleIcon)
	button.SpecTexture:SetTexture(playerdata.SpecIcon)

	RS_Template_SetSpellList(button.SpellList,playerdata.SpellList,playerdata.SelectedSpell)
end

RS.Template.CreatePlayerSpellList=function(parent,width,height,buttonheight,onspellclickfunc)
	local frame=RS.Template.CreateScrollFrame(parent,width,height)
	frame.PlayerList={}
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame.ButtonHeight=buttonheight

	frame:HookScript("OnMouseDown",function(...)
		parent:GetScript("OnMouseDown")(...)
	end)
	frame:HookScript("OnMouseUp",function(...)
		parent:GetScript("OnMouseUp")(...)
	end)
	
	return frame
end

RS.Template.UpdatePlayerSpellList=function(frame,playerdatalist)
	local playerlist=frame.PlayerList
	local buttonheight=frame.ButtonHeight
	local i=1
	for name,playerdata in pairs(playerdatalist)do
		if(not playerlist[i])then
			playerlist[i]=RS_Template_CreatePlayerFrame(frame,i)
		end
		playerframe=playerlist[i]

		RS_Template_SetPlayerFrame(playerframe,playerdata)
		playerframe:Show()
		i=i+1
	end
	for j=i ,table.getn(playerlist)do
		playerlist[j]:Hide()
	end

	frame.PlayerDataList=playerdatalist
end